unit sim06u1;

(*For tutorial ((WEBPAGE))-
  Some general tricks and techniques, illustrated along the way
  of simulating a simple woodland's population dynamics
  *)

(*Of interest only to me: Drafted 27 Jan 16, en route to BM, my
  C'mas present from D&ML. I was reading White's Selborne at the time,
  that may have triggered me to draft, yet again, an outline of sometbing
  fancier, during which I saw this "heart of the problem" clearly.
  *)

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs,
  StdCtrls, Clipbrd;

const vers='30 Jan 16';

type

  { Tsim06f1 }

  Tsim06f1 = class(TForm)
    buRestartSim: TButton;
    buQuit: TButton;
    buDoYear: TButton;
    buCopyURL: TButton;
    laClickToCopy: TLabel;
    laLinkToTutTxt: TLabel;
    laMemoColHeadings: TLabel;
    laBM_def: TLabel;
    meReportData: TMemo;
    procedure buCopyURLClick(Sender: TObject);
    procedure buQuitClick(Sender: TObject);
    procedure buDoYearClick(Sender: TObject);
    procedure buRestartSimClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure laClickToCopyClick(Sender: TObject);
  private
    { private declarations }
    lwBM_grass, lwBM_rabbit, lwBM_fox:longword;
    lwBM_grassPrev, lwBM_rabbitPrev, lwBM_foxPrev:longword;
    wYr,
      wGrassAD,wGrassRD,
      wRabbitAD,wRabbitRD,
      wFoxAD,wFoxRD          :word;
    procedure ReportBiomasses;
    procedure InitOnce;
    procedure InitToRestartSim;
    procedure AddToTopOfMemo(sLineToAddL:string);
    function sLeftPadded(lwToRender:longword;bHowMany:byte):string;
    procedure EstablishInitialRates;

    procedure DoYear;
    procedure MakeNoteOfPreviousBiomasses;
    procedure GrassAdd;
    procedure GrassReduce;
    procedure RabbitsAdd;
    procedure RabbitsReduce;
    procedure FoxesAdd;
    procedure FoxesReduce;

  public
    { public declarations }
  end;

var
  sim06f1: Tsim06f1;

implementation

{$R *.lfm}

{ Tsim06f1 }

procedure Tsim06f1.DoYear;
begin
  MakeNoteOfPreviousBiomasses;
  GrassAdd;
  GrassReduce;
  RabbitsAdd;
  RabbitsReduce;
  FoxesAdd;
  FoxesReduce;
  ReportBiomasses;
end;//DoYear

procedure Tsim06f1.GrassAdd;
begin
  lwBM_grass:=(lwBM_grassPrev*wGrassAD) div 1000;
end;//GrassAdd

procedure Tsim06f1.GrassReduce;
begin
  lwBM_grass:=lwBM_grassPrev-((lwBM_rabbit*wGrassRD) div 100);
end;//GrassReduce

procedure Tsim06f1.RabbitsAdd;
begin
  lwBM_rabbit:=(lwBM_rabbitPrev*wRabbitAD) div 1000;
end;//RabbitsAdd

procedure Tsim06f1.RabbitsReduce;
begin
  lwBM_rabbit:=lwBM_rabbitPrev-((lwBM_fox*wRabbitRD) div 100);
end;//RabbitsReduce

procedure Tsim06f1.FoxesAdd;
begin
  lwBM_fox:=(lwBM_foxPrev*wFoxAD) div 1000;
end;//FoxesAdd

procedure Tsim06f1.FoxesReduce;
begin
  lwBM_fox:=(lwBM_foxPrev*wFoxRD) div 1000;
end;//FoxesReduce

function Tsim06f1.sLeftPadded(lwToRender:longword;bHowMany:byte):string;
var sTmpL:string;
begin
  sTmpL:=inttostr(lwToRender);
  while NOT(length(sTmpL)>=bHowMany) do
    sTmpL:=' '+sTmpL;
  result:=sTmpL;
end;//sLeftPadded

  procedure Tsim06f1.AddToTopOfMemo(sLineToAddL:string);
var wTmpL:word;
begin
  meReportData.lines.insert(0,sLineToAddL);
end;//of AddToTopOfMemo

procedure Tsim06f1.InitOnce;
begin
 randomize;
 meReportData.lines.clear;
 AddToTopOfMemo('Welcome to sim06');
 AddToTopOfMemo('Version: '+vers);
 AddToTopOfMemo(' ');
 laMemoColHeadings.caption:=' Year   Grass BM   Rabbit BM   Fox BM';
end;//InitOnce;

procedure Tsim06f1.EstablishInitialRates;
//Some of these rates may be static, may remain unchanged
//  over the whole execution of the program's code.
//ppk: Parts per thousand. I.e. 1020 is equivalent to 102 percent
begin
  wGrassAD:=1020;//"Grass: Added to at rate (Delta) of xx ppk."
                 //(Grass growth per cycle.)
  wGrassRD:=2;//Reduction of grass per cycle, per 100 units of rabbit biomass

  wRabbitAD:=1040;
  wRabbitRD:=15;
  wFoxAD:=1400;
  wFoxRD:=998;

end;//EstablishInitialRates;

procedure Tsim06f1.InitToRestartSim;
begin
  lwBM_grass:=1000000;
  lwBM_rabbit:=20000;
  lwBM_fox:=200;
  wYr:=0;
  EstablishInitialRates;
  ReportBiomasses;
end;//InitToRestartSim;

procedure Tsim06f1.buQuitClick(Sender: TObject);
begin
  close;//terminate doesn't go through "close" event handler
end;

procedure Tsim06f1.buCopyURLClick(Sender: TObject);
(*With thanks to....
   http://wiki.lazarus.freepascal.org/Clipboard
   .... for how to do this. Note you must also put "Clipbrd" in
        your uses clause.*)
begin
// Clear any previous formats off the clipboard before starting
  Clipboard.Clear;
  Clipboard.AsText:='http://sheepdogguides.com/lut/lt3Ng-1.htm';
end;

procedure Tsim06f1.ReportBiomasses;
begin
  AddToTopOfMemo(sLeftPadded(wYr,4)+
     sLeftPadded(lwBM_grass,11)+
     sLeftPadded(lwBM_rabbit,11)+
     sLeftPadded(lwBM_fox,10));
end;// ReportBiomasses;

procedure Tsim06f1.MakeNoteOfPreviousBiomasses;
begin
  lwBM_grassPrev:=lwBM_grass;
  lwBM_rabbitPrev:=lwBM_rabbit;
  lwBM_foxPrev:=lwBM_fox;
end;//MakeNoteOfPreviousBiomasses

procedure Tsim06f1.buDoYearClick(Sender: TObject);
begin
  DoYear;
end;

procedure Tsim06f1.buRestartSimClick(Sender: TObject);
begin
  AddToTopOfMemo(' ');
  AddToTopOfMemo(' ');
  AddToTopOfMemo('Simulation restarted...');
  AddToTopOfMemo(' ');
  InitToRestartSim;
  ReportBiomasses;
  meReportData.VertScrollBar.Position:=0;//"should" work(?) but there's
  //a suggestion in a discussion online...
  //http://forum.lazarus.freepascal.org/index.php?topic=30468.0
  //that maybe this can't be used by most Lazarus people as of late 2015
  //NOT a critical element of the program... only causes problem if you
  //  scroll down after memo gets to big to show in window, and then
  //  do a reset, and aren't willing to rescroll to the top by hand.
end;

procedure Tsim06f1.FormActivate(Sender: TObject);
begin
  InitOnce;
  InitToRestartSim;
end;

procedure Tsim06f1.FormCreate(Sender: TObject);
begin
  //Shift running form up and left from were the in-compiler
  //  form is positioned, to make selecting either easy.
  top:=top-20;
  left:=left-20;

  sim06f1.caption:='sim06- framework for population dynamics model, and similar. Vers '+ vers;
  application.title:='sim06';

end;

procedure Tsim06f1.laClickToCopyClick(Sender: TObject);
begin

end;

end.

